
;;; Credit crunch model by Peter Thompson

__includes ["bdi.nls"]

globals [poundvalue stockvalue richamount richpay middleamount middlepay pooramount poorpay importpercent numofcollapsed]
breed [families family]
breed [banks bank]
banks-own [beliefs intentions reserves classtype bankx banky collapsed]
families-own [beliefs intentions banknumber class salary wallet account hungry breadtype homeposx homeposy loanday bankrupt]
patches-own []

to setup-families
 create-families people [
    set shape "person"
    rand-xy-co
    set color green
    set homeposx pxcor
    set homeposy pycor
    set beliefs ["bankcollapse" "false"]
    set intentions []
    set wallet random 10
    set account starting + random 1000 ; start with a varied amount in the account.
    set banknumber people + random numofbanks
    set breadtype -1
    if importpercent > 0 [
      set breadtype 0
      set importpercent importpercent - 1
    ]
    if breadtype = -1 and importpercent <= 0 [
      set breadtype 1
    ]
    set hungry 0
    set salary 0
    set class "none"
    if richamount > 0 [
      set salary richpay
      set richamount richamount - 1
      set class "rich"
    ]
    if salary = 0 and middleamount > 0 [
      set salary middlepay
      set middleamount middleamount - 1
      set class "middle"
    ]
    if salary = 0 and pooramount > 0 [
      set salary poorpay
      set pooramount pooramount - 1
      set class "poor"
    ]
    set loanday -1
    set bankrupt false
    
  ]
end

to setup-banks
  create-banks numofbanks 
  [
    set shape "house"
    rand-xy-co
    set bankx xcor
    set banky ycor
    setxy bankx banky
    set color white
    set beliefs []
    set intentions []
    set reserves random 50000 + 50000
    set collapsed false
  ]
end

to switchbank
  ; this gets called upon a belief that states that the bank is bust.
  ; from bust bank to one that is not.
  
  if wallet < 10 ;;;;;;;;;;;;; need a better fix here
  [
    set bankrupt true
    set color grey
  ]
  if wallet >= 10
  [

    if numofcollapsed < numofbanks and ticks < duration
    [ 
      find-collapsed
      let newbank (people)
      while [[collapsed] of bank (newbank) = false]
      [
        if newbank + 1 <= people + numofbanks
        [
          set newbank newbank + 1
        ]
        if newbank + 1 > people + numofbanks
        [
          set newbank (people)
        ]
      ]
      set banknumber (newbank)
      ; put what is in wallet into new bank
      set account wallet
      set wallet 0      
    ]
  ]

end

to bankrun

  let closeacc account
  let chance 100 / selfinterested
  let roll random chance
  
  if roll = 1 or selfinterested = 100
  [
    if [reserves] of bank banknumber >= closeacc
    [
      set wallet wallet + closeacc
      ask bank banknumber [set reserves reserves - closeacc]
      set account 0 
    ]
  ]

    
end

to buybread
  if wallet >= 2.5 ;;;;;;;;;;; THIS WILL BE REPLACED BY A BELIEF
  [
    if breadtype = 1 [ ; british bread
      set poundvalue poundvalue + 0.000001
      set wallet wallet - 2.5
      set hungry 0
    ]
    if breadtype = 0 [ ; imported bread
      set poundvalue poundvalue - 0.000005
      set wallet wallet - 1.8
      set hungry 0
    ]
  ]
end  


to withdraw

  ask bank banknumber [set reserves reserves - 10] 
  set wallet 10 
  set account account - 10
  if showwithdrawlinks = true 
  [
    create-link-to bank banknumber[set color red]
  ]

end

to deposit
  
  let amount salary
  ask bank banknumber [set reserves reserves + amount] 
  set account account + amount 
  if showdepositlinks = true 
  [
    create-link-to bank banknumber[set color green]
  ]

end

to loan

  ask bank banknumber [set reserves reserves - 200] 
  set account account + 200 
  set loanday ticks 

  create-link-to bank banknumber[set color blue]
  if showloanlabels = true
  [
    set label "loan"
  ]
  
  if loanday != -1 and ticks = loanday + 30 [
    if account >= 220 
    [
      ask bank banknumber[set reserves reserves + 220] 
      set account account - 220 
      set loanday -1
      set label ""
    ]
    if account < 220 
    [
      set bankrupt true
      set color grey
    ]
  ]
end


to rand-xy-co
  let x 0
  let y 0
  
  loop [
    set x random-pxcor
    set y random-pycor
    if not any? turtles-on patch x y and not (abs x < 4 and abs y < 4) [setxy x y stop]
  ]
end

to setup
  clear-all      ;; clear the world
  ;set duration 1187
  set poundvalue 1.0001
  set richpay 30
  set middlepay 7
  set poorpay 3
  set richamount people * 0.2
  set middleamount people * 0.6
  set pooramount people * 0.2
  set importpercent importpercentin
  set numofcollapsed 0
  setup-plot
  setup-families
  setup-banks
end

to go

  if ticks < duration 
  [
    ask banks [set reserves (reserves * (poundvalue - 1)) / 10 + reserves ]
      
    update-beliefs
    
    ask families [family-behaviour]
    ask families [set hungry hungry + 1] ; needs proper implementation
    ask banks [ask my-in-links [ die ]]
    
    update-plot
    tick ; Pass one day
  ]
  if numofcollapsed = numofbanks [stop]
  
end

to family-behaviour
  update-intentions
  execute-intentions ; many times for there may be many intentions on the stack
  execute-intentions
  execute-intentions
  execute-intentions
  execute-intentions
  execute-intentions
end

to find-collapsed

  let numofcollapsedcount 0
  ask banks 
  [
    if reserves <= 0
    [
      set pcolor red 
      set numofcollapsedcount numofcollapsedcount + 1 
      set collapsed true
    ]
  ]

  set numofcollapsed numofcollapsedcount
  if numofcollapsed = numofbanks [stop]
  
end

to update-beliefs
  let collapsedchange numofcollapsed
  find-collapsed
  if collapsedchange != numofcollapsed 
  [
    set poundvalue poundvalue - 0.0006
  ]
  
end

to update-intentions 
  
  if wallet >= 2.5
  [
    add-intention "buybread" "true"
  ]

  
  if bankrupt = false
  [
    if [reserves] of bank banknumber >= 0 and [collapsed] of bank banknumber = false 
    [
      if wallet < 2.5
      [
        add-intention "withdraw" "true"
      ]
    ]
  ]
  
  if bankrupt = false
  [
    if wallet >= 8 
    [
      add-intention "deposit" "true"
    ]
  ]
  
  if [collapsed] of bank banknumber = false
  [
    if account < 1000 and bankrupt = false 
    [
      add-intention "loan" "true"
    ]
  ]
    
  if [reserves] of bank banknumber <= 12000 
  [
    if account > 0
    [
      add-intention "bankrun" "true"
    ]
  ]
  if [collapsed] of bank banknumber = true
  [
    add-intention "switchbank" "true"
  ]
end

to setup-plot
  set-current-plot "Average Bank Reserves"
end

to update-plot
  set-current-plot-pen "reserves"
  plot mean [reserves] of banks
end
@#$#@#$#@
GRAPHICS-WINDOW
211
11
728
471
19
16
13.0
1
8
1
1
1
0
1
1
1
-19
19
-16
16
0
0
1
days

BUTTON
1
11
64
44
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
65
11
128
44
step
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
129
11
210
44
go (loop)
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

MONITOR
130
408
208
453
poundvalue
poundvalue
17
1
11

PLOT
729
11
996
471
Average Bank Reserves
Days
average reserves
0.0
10.0
0.0
10000.0
true
false
PENS
"reserves" 1.0 0 -10899396 false

SWITCH
1
278
144
311
show-intentions
show-intentions
1
1
-1000

SLIDER
1
44
173
77
people
people
50
450
309
1
1
NIL
HORIZONTAL

SLIDER
1
110
173
143
importpercentin
importpercentin
0
100
50
1
1
%
HORIZONTAL

SLIDER
1
77
173
110
numofbanks
numofbanks
1
30
24
1
1
NIL
HORIZONTAL

SLIDER
1
143
173
176
starting
starting
2500
3500
2500
10
1

HORIZONTAL

SWITCH
1
244
149
277
showdepositlinks
showdepositlinks
0
1
-1000

MONITOR
1
408
129
453
Collapsed banks
numofcollapsed
17
1
11

SWITCH
1
312
159
345
showwithdrawlinks
showwithdrawlinks
0
1
-1000

SWITCH
1
346
140
379
showloanlabels
showloanlabels
0
1
-1000

SLIDER
1
176
173
209
selfinterested
selfinterested
1
100
82
1
1
%
HORIZONTAL

SLIDER
173
44
206
194
duration
duration
1000
9000
6450
10
1
 days 
VERTICAL

@#$#@#$#@
WHAT IS IT?
-----------
I have decided to model the credit crunch from its early stages where the banks signed many irresponsible loans in order to boost its profits, through to where these loans defaulted, to where the bank's collapse because of the worsening economy and weakening of the pound. The dynamic model allows for many input scenarios where the culprits that caused the credit crunch can be identified and focussed on, to find solutions as to how the crisis could have been prevented, or indeed can be prevented from a specific moment onwards.


HOW IT WORKS
------------
The model view shows people and their banks interacting, red arrows showing withdrawals and green arrows showing deposits, blue arrows are loan agreements. If a person takes out a loan, they get a label of "loan" until this is paid off; the loan period is 30 days. 
Should a person become bankrupt, their figure becomes grey and no longer interacts with a bank, any loans they may have default.
Bank can collapse when they run out of reserves, and their patch colour (background) becomes red, there will be no more transactions with people once the bank has collapsed.
Although the term people has been used throughout the report,  agents can be referred to as families or even companies, which can be pushed into administration when their account is empty. Every one of these entities has a bank account with a specific starting value, they have a mentality to import food or buy British made food, their mentality can also be altered in terms of engaging in bank runs.

HOW TO USE IT
-------------
Select the sliders to the desired position, click setup and go (loop).


THINGS TO NOTICE
----------------
Fine tuning can provide many different models for the credit crunch.

THINGS TO TRY
-------------
People: 309
Banks: 24
Import percentage: 50%
Starting: 2500
Self Interested: 82%
Duration: 6000 days


EXTENDING THE MODEL
-------------------
We could extend the model by showing a recovery after the credit crunch. I would have like to have made the actions of agents more transparent to other agents, effectively modelling the credit crunch is a more effective manner as it is argued that the media and word of mouth played a big part in the crisis, resulting in problems such as less spending, spending on cheaper and imported goods, leading to less money being pumped into the economy, and engaging in bank runs collapsing many banks.



CREDITS AND REFERENCES
----------------------
[1] Anand S. Rao Australian Artificial Intelligence Institute
Michael P. George.  BDI Agents From Theory to Practice http://www.agent.ai/doc/upload/200302/rao95.pdf

[2] Ilias Sakellariou NetLogoBDI Extension  http://users.uom.gr/\~iliass/projects/NetLogo/

[3] Wilensky, U. (1999). NetLogo. http://ccl.northwestern.edu/netlogo. Center for Connected Learning and Computer-Based Modeling. Northwestern University, Evanston, IL.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
0
Rectangle -7500403 true true 151 225 180 285
Rectangle -7500403 true true 47 225 75 285
Rectangle -7500403 true true 15 75 210 225
Circle -7500403 true true 135 75 150
Circle -16777216 true false 165 76 116

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
